;;;; -*- Mode: Lisp; Syntax: ANSI-Common-Lisp -*-

#|
Copyright (C) 2007  David Owen <dsowen@fugue88.ws>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
|#

(in-package #:dso-parse)



(defmacro defparser (name rule &optional filter)
  "Defines a function named NAME that parses its input according to
RULE (see IF-MATCHES).  On success, returns the values T, the
unconsumed input, and the tree (labeled with NAME).  On failure ,
returns NIL.

The tree may optionally be filtered by FILTER before being labeled."
  `(defun ,name (input)
    (if-matches input ,rule (next match)
     (values t next (list ',name ,(if filter `(,filter match) 'match))))))

(defmacro defgrammar (() &body definitions)
  "Defines a grammar, being a collection of parsers.  The syntax is:

(DEFGRAMMAR (options*) definitions*)
definitions: (name rule filter?)"
  (flet ((x (definition)
	   (destructuring-bind (name rule &optional filter) definition
	     `(defparser ,name ,rule ,filter))))
    `(progv () ,@(mapcar #'x definitions))))
